@@ -97,7 +97,7 @@ public:
   AggDevice(const char* fp, int w, int h, double ps, int bg, double res, 
             double scaling);
   virtual ~AggDevice();
-  void newPage(unsigned int bg);
+  virtual void newPage(unsigned int bg);
   void close();
   virtual bool savePage();
   SEXP capture();
@@ -136,7 +136,7 @@ public:
   void drawText(double x, double y, const char *str, const char *family, 
                 int face, double size, double rot, double hadj, int col);
   
-private:
+protected:
   virtual inline R_COLOR convertColour(unsigned int col) {
     return R_COLOR(R_RED(col), R_GREEN(col), R_BLUE(col), R_ALPHA(col)).premultiply();
   }
@@ -185,11 +185,12 @@ private:
     }
   }
   template<class Raster, class Path>
-  void setStroke(Raster &ras, Path &p, int lty, double lwd, R_GE_lineend lend, R_GE_linejoin ljoin) {
+  void setStroke(Raster &ras, Path &p, int lty, double lwd, R_GE_lineend lend, R_GE_linejoin ljoin, double lmitre) {
     if (lty == LTY_SOLID) {
       agg::conv_stroke<Path> pg(p);
       pg.width(lwd);
       pg.line_join(convertLinejoin(ljoin));
+      pg.miter_limit(lmitre);
       pg.line_cap(convertLineend(lend));
       ras.add_path(pg);
     } else {
@@ -198,6 +199,7 @@ private:
       makeDash(pd, lty, lwd);
       pg.width(lwd);
       pg.line_join(convertLinejoin(ljoin));
+      pg.miter_limit(lmitre);
       pg.line_cap(convertLineend(lend));
       ras.add_path(pg);
     }
@@ -231,7 +233,8 @@ private:
   template<class Raster, class Path>
   void drawShape(Raster &ras, Raster &ras_clip, Path &path, bool draw_fill, 
                  bool draw_stroke, int fill, int col, double lwd, 
-                 int lty, R_GE_lineend lend, R_GE_linejoin ljoin = GE_ROUND_JOIN, int pattern = -1, bool evenodd = false) {
+                 int lty, R_GE_lineend lend, R_GE_linejoin ljoin = GE_ROUND_JOIN, 
+                 double lmitre = 1.0, int pattern = -1, bool evenodd = false) {
     agg::scanline_p8 slp;
     if (recording_clip != NULL) {
       recording_clip->concat_path(path);
@@ -283,7 +286,7 @@ private:
     
     if (evenodd) ras.filling_rule(agg::fill_non_zero);
     agg::scanline_u8 slu;
-    setStroke(ras, path, lty, lwd, lend, ljoin);
+    setStroke(ras, path, lty, lwd, lend, ljoin, lmitre);
     if (recording_mask == NULL && recording_pattern == NULL) {
       solid_renderer.color(convertColour(col));
       if (current_mask == NULL) {
@@ -775,7 +778,7 @@ void AggDevice<PIXFMT, R_COLOR, BLNDFMT>::drawCircle(double x, double y, double
     e1.init(x, y, r, r);
   }
   
-  drawShape(ras, ras_clip, e1, draw_fill, draw_stroke, fill, col, lwd, lty, lend, GE_ROUND_JOIN, pattern);
+  drawShape(ras, ras_clip, e1, draw_fill, draw_stroke, fill, col, lwd, lty, lend, GE_ROUND_JOIN, 1.0, pattern);
 }
 
 template<class PIXFMT, class R_COLOR, typename BLNDFMT>
@@ -805,7 +808,7 @@ void AggDevice<PIXFMT, R_COLOR, BLNDFMT>::drawRect(double x0, double y0, double
   rect.line_to(x1, y0);
   rect.close_polygon();
   
-  drawShape(ras, ras_clip, rect, draw_fill, draw_stroke, fill, col, lwd, lty, lend, GE_ROUND_JOIN, pattern);
+  drawShape(ras, ras_clip, rect, draw_fill, draw_stroke, fill, col, lwd, lty, lend, GE_ROUND_JOIN, 1.0, pattern);
 }
 
 template<class PIXFMT, class R_COLOR, typename BLNDFMT>
@@ -832,7 +835,7 @@ void AggDevice<PIXFMT, R_COLOR, BLNDFMT>::drawPolygon(int n, double *x, double *
   }
   poly.close_polygon();
   
-  drawShape(ras, ras_clip, poly, draw_fill, draw_stroke, fill, col, lwd, lty, lend, ljoin, pattern);
+  drawShape(ras, ras_clip, poly, draw_fill, draw_stroke, fill, col, lwd, lty, lend, ljoin, lmitre, pattern);
 }
 
 template<class PIXFMT, class R_COLOR, typename BLNDFMT>
@@ -874,7 +877,7 @@ void AggDevice<PIXFMT, R_COLOR, BLNDFMT>::drawPolyline(int n, double* x, double*
     ps.line_to(x[i]  + x_trans, y[i] + y_trans);
   }
   
-  drawShape(ras, ras_clip, ps, false, true, 0, col, lwd, lty, lend, ljoin);
+  drawShape(ras, ras_clip, ps, false, true, 0, col, lwd, lty, lend, ljoin, lmitre);
 }
 
 template<class PIXFMT, class R_COLOR, typename BLNDFMT>
@@ -912,7 +915,7 @@ void AggDevice<PIXFMT, R_COLOR, BLNDFMT>::drawPath(int npoly, int* nper, double*
     path.close_polygon();
   }
   
-  drawShape(ras, ras_clip, path, draw_fill, draw_stroke, fill, col, lwd, lty, lend, ljoin, pattern, evenodd);
+  drawShape(ras, ras_clip, path, draw_fill, draw_stroke, fill, col, lwd, lty, lend, ljoin, lmitre, pattern, evenodd);
 }
 
 template<class PIXFMT, class R_COLOR, typename BLNDFMT>
